@@ -2,10 +2,10 @@ import os, argparse, glob, pathlib, time
 import subprocess
 import numpy as np
 from natsort import natsorted
+from tqdm import tqdm
 
 from . import utils, models, io
 
-
 try:
     from cellpose import gui 
     GUI_ENABLED = True 
@@ -22,13 +22,17 @@ except Exception as err:
 def main():
     parser = argparse.ArgumentParser(description='cellpose parameters')
     parser.add_argument('--check_mkl', action='store_true', help='check if mkl working')
-    parser.add_argument('--mkldnn', action='store_true', help='force MXNET_SUBGRAPH_BACKEND = "MKLDNN"')
+    parser.add_argument('--mkldnn', action='store_true', help='for mxnet, force MXNET_SUBGRAPH_BACKEND = "MKLDNN"')
     parser.add_argument('--train', action='store_true', help='train network using images in dir')
     parser.add_argument('--dir', required=False, 
                         default=[], type=str, help='folder containing data to run or train on')
+    parser.add_argument('--mxnet', action='store_true', help='use mxnet')
     parser.add_argument('--img_filter', required=False, 
                         default=[], type=str, help='end string for images to run on')
     parser.add_argument('--use_gpu', action='store_true', help='use gpu if mxnet with cuda installed')
+    parser.add_argument('--fast_mode', action='store_true', help="make code run faster by turning off 4 network averaging")
+    parser.add_argument('--resample', action='store_true', help="run dynamics on full image (slower for images with large diameters)")
+    parser.add_argument('--no_interp', action='store_true', help='do not interpolate when running dynamics (was default)')
     parser.add_argument('--do_3D', action='store_true',
                         help='process images as 3D stacks of images (nplanes x nchan x Ly x Lx')
     # settings for running cellpose
@@ -51,7 +55,6 @@ def main():
                         default=0.0, type=float, help='cell probability threshold, centered at 0.0')
     parser.add_argument('--save_png', action='store_true', help='save masks as png and outlines as text file for ImageJ')
     parser.add_argument('--save_tif', action='store_true', help='save masks as tif and outlines as text file for ImageJ')
-    parser.add_argument('--fast_mode', action='store_true', help="make code run faster by turning off 4 network averaging")
     parser.add_argument('--no_npy', action='store_true', help='suppress saving of npy')
 
     # settings for training
@@ -73,11 +76,10 @@ def main():
     parser.add_argument('--concatenation', required=False, 
                         default=0, type=int, help='concatenate downsampled layers with upsampled layers (off by default which means they are added)')
 
-
     args = parser.parse_args()
 
     if args.check_mkl:
-        mkl_enabled = utils.check_mkl()
+        mkl_enabled = models.check_mkl((not args.mxnet))
     else:
         mkl_enabled = True
 
@@ -86,8 +88,6 @@ def main():
     else:
         os.environ["MXNET_SUBGRAPH_BACKEND"]=""
     
-    import mxnet as mx
-
     if len(args.dir)==0:
         if not GUI_ENABLED:
             print('ERROR: %s'%GUI_ERROR)
@@ -107,16 +107,9 @@ def main():
         else:
             imf = None
 
-        
-        
-        if args.use_gpu:
-            use_gpu = utils.use_gpu()
-        if use_gpu:
-            device = mx.gpu()
-        else:
-            device = mx.cpu()
-        print('>>>> using %s'%(['CPU', 'GPU'][use_gpu]))
-        model_dir = pathlib.Path.home().joinpath('.cellpose', 'models')              
+
+        device, gpu = models.assign_device((not args.mxnet), args.use_gpu)
+        model_dir = models.model_dir              
 
         if not args.train and not args.train_size:
             tic = time.time()
@@ -128,57 +121,52 @@ def main():
 
             image_names = io.get_image_files(args.dir, args.mask_filter, imf=imf)
             nimg = len(image_names)
-            images = [io.imread(image_names[n]) for n in range(nimg)]
-
-            if args.pretrained_model=='cyto' or args.pretrained_model=='nuclei':
-                model = models.Cellpose(device=device, model_type=args.pretrained_model)
-                    
-                if args.diameter==0:
+            if args.diameter==0:
+                if args.pretrained_model=='cyto' or args.pretrained_model=='nuclei':
                     diameter = None
                     print('>>>> estimating diameter for each image')
                 else:
-                    diameter = args.diameter
-                    print('>>>> using diameter %0.2f for all images'%diameter)
-
-                cstr0 = ['GRAY', 'RED', 'GREEN', 'BLUE']
-                cstr1 = ['NONE', 'RED', 'GREEN', 'BLUE']
-                print('running cellpose on %d images using chan_to_seg %s and chan (opt) %s'%
-                        (nimg, cstr0[channels[0]], cstr1[channels[1]]))
-                
-                masks, flows, _, diams = model.eval(images, channels=channels, diameter=diameter,
-                                                    do_3D=args.do_3D, net_avg=(not args.fast_mode),
-                                                    augment=False,
-                                                    flow_threshold=args.flow_threshold,
-                                                    cellprob_threshold=args.cellprob_threshold,
-                                                    batch_size=args.batch_size)
+                    print('>>>> using user-specified model, no auto-diameter estimation available')
+                    diameter = model.diam_mean
+            else:
+                diameter = args.diameter
+                print('>>>> using diameter %0.2f for all images'%diameter)
                 
+            cstr0 = ['GRAY', 'RED', 'GREEN', 'BLUE']
+            cstr1 = ['NONE', 'RED', 'GREEN', 'BLUE']
+            print('>>>> running cellpose on %d images using chan_to_seg %s and chan (opt) %s'%
+                            (nimg, cstr0[channels[0]], cstr1[channels[1]]))
+                    
+            if args.pretrained_model=='cyto' or args.pretrained_model=='nuclei':
+                model = models.Cellpose(gpu=gpu, device=device, model_type=args.pretrained_model, 
+                                            torch=(not args.mxnet))
             else:
                 if args.all_channels:
                     channels = None  
-                model = models.CellposeModel(device=device, pretrained_model=cpmodel_path,
-                                             batch_size=args.batch_size)
-                if args.diameter==0:
-                    print('>>>> using user-specified model, no auto-diameter estimation available')
-                    diameter = model.diam_mean
-                else:
-                    diameter = args.diameter
-                rescale = model.diam_mean / diameter
-                masks, flows, _ = model.eval(images, channels=channels, rescale=rescale,
-                                             do_3D=args.do_3D,
-                                             augment=False,
-                                             flow_threshold=args.flow_threshold,
-                                             cellprob_threshold=args.cellprob_threshold)
-                diams = diameter * np.ones(len(images)) 
-                  
-            print('>>>> saving results')
-            if not args.no_npy:
-                io.masks_flows_to_seg(images, masks, flows, diams, image_names, channels)
-            if args.save_png or args.save_tif:
-                io.save_masks(images, masks, flows, image_names, png=args.save_png, tif=args.save_tif)
+                model = models.CellposeModel(gpu=gpu, device=device,
+                                             pretrained_model=cpmodel_path,
+                                             torch=(not args.mxnet))
+                
+            for image_name in tqdm(image_names):
+                image = io.imread(image_name)
+                masks, flows, _, diams = model.eval(image, channels=channels, diameter=diameter,
+                                                    do_3D=args.do_3D, net_avg=(not args.fast_mode),
+                                                    augment=False,
+                                                    resample=args.resample,
+                                                    flow_threshold=args.flow_threshold,
+                                                    cellprob_threshold=args.cellprob_threshold,
+                                                    batch_size=args.batch_size,
+                                                    interp=(not args.no_interp))
+                    
+                if not args.no_npy:
+                    io.masks_flows_to_seg(image, masks, flows, diams, image_name, channels)
+                if args.save_png or args.save_tif:
+                    io.save_masks(image, masks, flows, image_name, png=args.save_png, tif=args.save_tif)
             print('>>>> completed in %0.3f sec'%(time.time()-tic))
         else:
             if args.pretrained_model=='cyto' or args.pretrained_model=='nuclei':
-                cpmodel_path = os.fspath(model_dir.joinpath('%s_0'%(args.pretrained_model)))
+                torch_str = ['torch', '']
+                cpmodel_path = os.fspath(model_dir.joinpath('%s%s_0'%(args.pretrained_model, torch_str[args.mxnet])))
                 if args.pretrained_model=='cyto':
                     szmean = 30.
                 else:
@@ -218,7 +206,7 @@ def main():
                 
             # initialize model
             if args.unet:
-                model = models.UnetModel(device=device,
+                model = core.UnetModel(device=device,
                                         pretrained_model=cpmodel_path, 
                                         diam_mean=szmean,
                                         residual_on=args.residual_on,
@@ -227,6 +215,7 @@ def main():
                                         nclasses=args.nclasses)
             else:
                 model = models.CellposeModel(device=device,
+                                            torch=(not args.mxnet),
                                             pretrained_model=cpmodel_path, 
                                             diam_mean=szmean,
                                             residual_on=args.residual_on,
@@ -238,13 +227,15 @@ def main():
                 cpmodel_path = model.train(images, labels, train_files=image_names, 
                                             test_data=test_images, test_labels=test_labels, test_files=image_names_test,
                                             learning_rate=args.learning_rate, channels=channels, 
-                                            save_path=os.path.realpath(args.dir), rescale=rescale, n_epochs=args.n_epochs)
+                                            save_path=os.path.realpath(args.dir), rescale=rescale, n_epochs=args.n_epochs,
+                                            batch_size=args.batch_size)
+                model.pretrained_model = cpmodel_path
                 print('>>>> model trained and saved to %s'%cpmodel_path)
 
             # train size model
             if args.train_size:
-                sz_model = models.SizeModel(model, device=device)
-                sz_model.train(images, labels, test_images, test_labels, channels=channels)
+                sz_model = models.SizeModel(cp_model=model, device=device)
+                sz_model.train(images, labels, test_images, test_labels, channels=channels, batch_size=args.batch_size)
                 if test_images is not None:
                     predicted_diams, diams_style = sz_model.eval(test_images, channels=channels)
                     if test_labels[0].ndim>2:
