@@ -11,9 +11,6 @@ import numpy as np
 import cv2
 from scipy.ndimage import gaussian_filter
 
-import mxnet as mx
-from mxnet import nd
-
 from . import utils, transforms, models, guiparts, plot, menus, io, dynamics
 
 try:
@@ -91,6 +88,16 @@ def make_bwr():
     bwr = pg.ColorMap(pos=np.linspace(0.0,255,256), color=color)
     return bwr
 
+def make_spectral():
+    # make spectral colormap
+    r = np.array([0,4,8,12,16,20,24,28,32,36,40,44,48,52,56,60,64,68,72,76,80,84,88,92,96,100,104,108,112,116,120,124,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,120,112,104,96,88,80,72,64,56,48,40,32,24,16,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,7,11,15,19,23,27,31,35,39,43,47,51,55,59,63,67,71,75,79,83,87,91,95,99,103,107,111,115,119,123,127,131,135,139,143,147,151,155,159,163,167,171,175,179,183,187,191,195,199,203,207,211,215,219,223,227,231,235,239,243,247,251,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255])
+    g = np.array([0,1,2,3,4,5,6,7,8,9,10,9,9,8,8,7,7,6,6,5,5,5,4,4,3,3,2,2,1,1,0,0,0,7,15,23,31,39,47,55,63,71,79,87,95,103,111,119,127,135,143,151,159,167,175,183,191,199,207,215,223,231,239,247,255,247,239,231,223,215,207,199,191,183,175,167,159,151,143,135,128,129,131,132,134,135,137,139,140,142,143,145,147,148,150,151,153,154,156,158,159,161,162,164,166,167,169,170,172,174,175,177,178,180,181,183,185,186,188,189,191,193,194,196,197,199,201,202,204,205,207,208,210,212,213,215,216,218,220,221,223,224,226,228,229,231,232,234,235,237,239,240,242,243,245,247,248,250,251,253,255,251,247,243,239,235,231,227,223,219,215,211,207,203,199,195,191,187,183,179,175,171,167,163,159,155,151,147,143,139,135,131,127,123,119,115,111,107,103,99,95,91,87,83,79,75,71,67,63,59,55,51,47,43,39,35,31,27,23,19,15,11,7,3,0,8,16,24,32,41,49,57,65,74,82,90,98,106,115,123,131,139,148,156,164,172,180,189,197,205,213,222,230,238,246,254])
+    b = np.array([0,7,15,23,31,39,47,55,63,71,79,87,95,103,111,119,127,135,143,151,159,167,175,183,191,199,207,215,223,231,239,247,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,251,247,243,239,235,231,227,223,219,215,211,207,203,199,195,191,187,183,179,175,171,167,163,159,155,151,147,143,139,135,131,128,126,124,122,120,118,116,114,112,110,108,106,104,102,100,98,96,94,92,90,88,86,84,82,80,78,76,74,72,70,68,66,64,62,60,58,56,54,52,50,48,46,44,42,40,38,36,34,32,30,28,26,24,22,20,18,16,14,12,10,8,6,4,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,8,16,24,32,41,49,57,65,74,82,90,98,106,115,123,131,139,148,156,164,172,180,189,197,205,213,222,230,238,246,254])
+    color = (np.vstack((r,g,b)).T).astype(np.uint8)
+    spectral = pg.ColorMap(pos=np.linspace(0.0,255,256), color=color)
+    return spectral
+    
+
 def make_cmap(cm=0):
     # make a single channel colormap
     r = np.arange(0,256)
@@ -194,6 +201,9 @@ class MainW(QtGui.QMainWindow):
         bwrmap = make_bwr()
         self.bwr = bwrmap.getLookupTable(start=0.0, stop=255.0, alpha=False)
         self.cmap = []
+        # spectral colormap
+        self.cmap.append(make_spectral().getLookupTable(start=0.0, stop=255.0, alpha=False))
+        # single channel colormaps
         for i in range(3):
             self.cmap.append(make_cmap(i).getLookupTable(start=0.0, stop=255.0, alpha=False))
 
@@ -260,7 +270,7 @@ class MainW(QtGui.QMainWindow):
         self.color = 0 # 0=RGB, 1=gray, 2=R, 3=G, 4=B
         self.RGBChoose = guiparts.RGBRadioButtons(self, b,1)
         self.RGBDropDown = QtGui.QComboBox()
-        self.RGBDropDown.addItems(["RGB","gray","red","green","blue"])
+        self.RGBDropDown.addItems(["RGB","gray","spectral","red","green","blue"])
         self.RGBDropDown.setFont(self.medfont)
         self.RGBDropDown.currentIndexChanged.connect(self.color_choose)
         self.RGBDropDown.setFixedWidth(60)
@@ -402,9 +412,9 @@ class MainW(QtGui.QMainWindow):
 
         # fast mode
         self.NetAvg = QtGui.QComboBox()
-        self.NetAvg.addItems(['average 4 nets', 'run 1 net (fast)'])
+        self.NetAvg.addItems(['average 4 nets', '+ resample (slow)', 'run 1 net (fast)', ])
         self.NetAvg.setFont(self.medfont)
-        self.NetAvg.setToolTip('average 4 different fit networks or run 1 network to <i>increase</i> run speed')
+        self.NetAvg.setToolTip('average 4 different fit networks (default) + resample for smooth masks (slow) or run 1 network (fast)')
         self.l0.addWidget(self.NetAvg, b,1,1,1)
 
         b+=1
@@ -615,10 +625,10 @@ class MainW(QtGui.QMainWindow):
 
                 # can change background or stroke size if cell not finished
                 if event.key() == QtCore.Qt.Key_Up or event.key() == QtCore.Qt.Key_W:
-                    self.color = (self.color-1)%(5)
+                    self.color = (self.color-1)%(6)
                     self.RGBDropDown.setCurrentIndex(self.color)
                 elif event.key() == QtCore.Qt.Key_Down or event.key() == QtCore.Qt.Key_S:
-                    self.color = (self.color+1)%(5)
+                    self.color = (self.color+1)%(6)
                     self.RGBDropDown.setCurrentIndex(self.color)
                 elif (event.key() == QtCore.Qt.Key_Comma or
                         event.key() == QtCore.Qt.Key_Period):
@@ -640,13 +650,30 @@ class MainW(QtGui.QMainWindow):
         if event.key() == QtCore.Qt.Key_Minus or event.key() == QtCore.Qt.Key_Equal:
             self.p0.keyPressEvent(event)
 
-    def check_gpu(self):
-        if utils.use_gpu():
+    def check_gpu(self, torch=True):
+        # also decide whether or not to use torch
+        self.torch = torch
+        self.useGPU.setChecked(False)
+        self.useGPU.setEnabled(False)    
+        if self.torch and models.use_gpu(istorch=True):
             self.useGPU.setEnabled(True)
             self.useGPU.setChecked(True)
+        elif models.MXNET_ENABLED:
+            if models.use_gpu(istorch=False):
+                print('>>> will run model on GPU in mxnet <<<')
+                self.torch = False
+                self.useGPU.setEnabled(True)
+                self.useGPU.setChecked(True)
+            elif models.check_mkl(istorch=False):
+                print('>>> will run model on CPU (MKL-accelerated) in mxnet <<<')
+                self.torch = False
+                self.useGPU.setStyleSheet("color: rgb(80,80,80);")
+            elif not self.torch:
+                print('>>> will run model on CPU (not MKL-accelerated) in mxnet <<<')
+                self.useGPU.setStyleSheet("color: rgb(80,80,80);")
+            else:
+                self.useGPU.setStyleSheet("color: rgb(80,80,80);")
         else:
-            self.useGPU.setChecked(False)
-            self.useGPU.setEnabled(False)
             self.useGPU.setStyleSheet("color: rgb(80,80,80);")
 
     def get_channels(self):
@@ -855,6 +882,7 @@ class MainW(QtGui.QMainWindow):
             self.p0.removeItem(self.hLine)
 
     def clear_all(self):
+        self.prev_selected = 0
         self.selected = 0
         #self.layers_undo, self.cellpix_undo, self.outpix_undo = [],[],[]
         self.layers = 0*np.ones((self.NZ,self.Ly,self.Lx,4), np.uint8)
@@ -867,6 +895,7 @@ class MainW(QtGui.QMainWindow):
         self.update_plot()
 
     def select_cell(self, idx):
+        self.prev_selected = self.selected
         self.selected = idx
         if self.selected > 0:
             self.layers[self.cellpix==idx] = np.array([255,255,255,self.opacity])
@@ -914,6 +943,38 @@ class MainW(QtGui.QMainWindow):
         if self.NZ==1:
             io._save_sets(self)
 
+    def merge_cells(self, idx):
+        self.prev_selected = self.selected
+        self.selected = idx
+        if self.selected != self.prev_selected:
+            for z in range(self.NZ):
+                ar0, ac0 = np.nonzero(self.cellpix[z]==self.prev_selected)
+                ar1, ac1 = np.nonzero(self.cellpix[z]==self.selected)
+                touching = np.logical_and((ar0[:,np.newaxis] - ar1)==1,
+                                            (ac0[:,np.newaxis] - ac1)==1).sum()
+                print(touching)
+                ar = np.hstack((ar0, ar1))
+                ac = np.hstack((ac0, ac1))
+                if touching:
+                    mask = np.zeros((np.ptp(ar)+4, np.ptp(ac)+4), np.uint8)
+                    mask[ar-ar.min()+2, ac-ac.min()+2] = 1
+                    contours = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_NONE)
+                    pvc, pvr = contours[-2][0].squeeze().T            
+                    vr, vc = pvr + ar.min() - 2, pvc + ac.min() - 2
+                else:
+                    vr0, vc0 = np.nonzero(self.outpix[z]==self.prev_selected)
+                    vr1, vc1 = np.nonzero(self.outpix[z]==self.selected)
+                    vr = np.hstack((vr0, vr1))
+                    vc = np.hstack((vc0, vc1))
+                color = self.cellcolors[self.prev_selected]
+                self.draw_mask(z, ar, ac, vr, vc, color, idx=self.prev_selected)
+            self.remove_cell(self.selected)
+            print('merged two cells')
+            self.update_plot()
+            io._save_sets(self)
+            self.undo.setEnabled(False)      
+            self.redo.setEnabled(False)    
+
     def undo_remove_cell(self):
         if len(self.removed_cell) > 0:
             z = 0
@@ -1006,8 +1067,11 @@ class MainW(QtGui.QMainWindow):
             elif self.color==1:
                 image = image.astype(np.float32).mean(axis=-1).astype(np.uint8)
                 self.img.setImage(image, autoLevels=False, lut=None)
-            elif self.color>1:
-                image = image[:,:,self.color-2]
+            elif self.color==2:
+                image = image.astype(np.float32).mean(axis=-1).astype(np.uint8)
+                self.img.setImage(image, autoLevels=False, lut=self.cmap[0])
+            elif self.color>2:
+                image = image[:,:,self.color-3]
                 self.img.setImage(image, autoLevels=False, lut=self.cmap[self.color-2])
             self.img.setLevels(self.saturation[self.currentZ])
         else:
@@ -1076,7 +1140,7 @@ class MainW(QtGui.QMainWindow):
             ar, ac = ar+vr.min()-2, ac+vc.min()-2
             # get dense outline
             contours = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_NONE)
-            pvc, pvr = contours[0][0].squeeze().T            
+            pvc, pvr = contours[-2][0].squeeze().T            
             vr, vc = pvr + vr.min() - 2, pvc + vc.min() - 2
             # concatenate all points
             ar, ac = np.hstack((np.vstack((vr, vc)), np.vstack((ar, ac))))
@@ -1091,7 +1155,7 @@ class MainW(QtGui.QMainWindow):
                 mask = np.zeros((np.ptp(ar)+4, np.ptp(ac)+4), np.uint8)
                 mask[ar-ar.min()+2, ac-ac.min()+2] = 1
                 contours = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_NONE)
-                pvc, pvr = contours[0][0].squeeze().T            
+                pvc, pvr = contours[-2][0].squeeze().T            
                 vr, vc = pvr + ar.min() - 2, pvc + ac.min() - 2
             self.draw_mask(z, ar, ac, vr, vc, color)
 
@@ -1122,11 +1186,13 @@ class MainW(QtGui.QMainWindow):
 
         return median
 
-    def draw_mask(self, z, ar, ac, vr, vc, color):
+    def draw_mask(self, z, ar, ac, vr, vc, color, idx=None):
         ''' draw single mask using outlines and area '''
-        self.cellpix[z][vr, vc] = self.ncells+1
-        self.cellpix[z][ar, ac] = self.ncells+1
-        self.outpix[z][vr, vc] = self.ncells+1
+        if idx is None:
+            idx = self.ncells+1
+        self.cellpix[z][vr, vc] = idx
+        self.cellpix[z][ar, ac] = idx
+        self.outpix[z][vr, vc] = idx
         if self.masksOn:
             self.layers[z][ar, ac, :3] = color
             self.layers[z][ar, ac, -1] = self.opacity
@@ -1191,24 +1257,11 @@ class MainW(QtGui.QMainWindow):
         return image
 
     def initialize_model(self):
-        if self.useGPU.isChecked():
-            device = mx.gpu()
-        else:
-            device = mx.cpu()
-
-        change=False
-        if not hasattr(self, 'model') or self.ModelChoose.currentText() != self.current_model:
-            self.current_model = self.ModelChoose.currentText()
-            change=True
-        elif ((self.model.device==mx.gpu() and not self.useGPU.isChecked()) or
-                (self.model.device==mx.cpu() and self.useGPU.isChecked())):
-            # if device has changed, reload model
-            self.current_model = self.ModelChoose.currentText()
-            change=True
-
-        if change:
-            print(self.current_model)
-            self.model = models.Cellpose(device=device, model_type=self.current_model)
+        self.current_model = self.ModelChoose.currentText()
+        print(self.current_model)
+        self.model = models.Cellpose(gpu=self.useGPU.isChecked(), 
+                                     torch=self.torch,
+                                     model_type=self.current_model)
 
     def compute_cprob(self):
         rerun = False
@@ -1264,10 +1317,11 @@ class MainW(QtGui.QMainWindow):
             channels = self.get_channels()
             self.diameter = float(self.Diameter.text())
             try:
-                net_avg = 1 - self.NetAvg.currentIndex()
+                net_avg = self.NetAvg.currentIndex()<2
+                resample = self.NetAvg.currentIndex()==1
                 masks, flows, _, _ = self.model.eval(data, channels=channels,
                                                     diameter=self.diameter, invert=self.invert.isChecked(),
-                                                    net_avg=net_avg, augment=False,
+                                                    net_avg=net_avg, augment=False, resample=resample,
                                                     do_3D=do_3D, progress=self.progress)
             except Exception as e:
                 print('NET ERROR: %s'%e)
@@ -1279,20 +1333,19 @@ class MainW(QtGui.QMainWindow):
             #if not do_3D:
             #    masks = masks[0][np.newaxis,:,:]
             #    flows = flows[0]
+            self.flows[0] = flows[0].copy()
+            self.flows[1] = (np.clip(utils.normalize99(flows[2]),0,1) * 255).astype(np.uint8)
             if not do_3D:
                 masks = masks[np.newaxis,...]
-            self.flows[0] = transforms.resize_image(flows[0].copy(), masks.shape[-2], masks.shape[-1],
-                                                    interpolation=cv2.INTER_NEAREST)
-            self.flows[1] = (np.clip(utils.normalize99(flows[2]),0,1) * 255).astype(np.uint8)
-            self.flows[1] = transforms.resize_image(self.flows[1], masks.shape[-2], masks.shape[-1],
-                                                    interpolation=cv2.INTER_NEAREST)
+                self.flows[0] = transforms.resize_image(self.flows[0], masks.shape[-2], masks.shape[-1],
+                                                        interpolation=cv2.INTER_NEAREST)
+                self.flows[1] = transforms.resize_image(self.flows[1], masks.shape[-2], masks.shape[-1])
             if not do_3D:
                 self.flows[2] = np.zeros(masks.shape[1:], dtype=np.uint8)
                 self.flows = [self.flows[n][np.newaxis,...] for n in range(len(self.flows))]
             else:
                 self.flows[2] = (flows[1][0]/10 * 127 + 127).astype(np.uint8)
-                self.flows[2] = transforms.resize_image(self.flows[2], masks.shape[-2], masks.shape[-1],
-                                                        interpolation=cv2.INTER_NEAREST)
+                
             if len(flows)>2:
                 self.flows.append(flows[3])
                 self.flows.append(np.concatenate((flows[1], flows[2][np.newaxis,...]), axis=0))
