@@ -6,19 +6,23 @@ import cv2
 from scipy.ndimage import find_objects, gaussian_filter, generate_binary_structure, label, maximum_filter1d, binary_fill_holes
 from scipy.spatial import ConvexHull
 import numpy as np
-import mxnet as mx
+import colorsys
 
 from . import metrics
 
-def use_gpu(gpu_number=0):
-    """ check if mxnet gpu works """
-    try:
-        _ = mx.ndarray.array([1, 2, 3], ctx=mx.gpu(gpu_number))
-        print('** CUDA version installed and working. **')
-        return True
-    except mx.MXNetError:
-        print('CUDA version not installed/working, will use CPU version.')
-        return False
+def rgb_to_hsv(arr):
+    rgb_to_hsv_channels = np.vectorize(colorsys.rgb_to_hsv)
+    r, g, b = np.rollaxis(arr, axis=-1)
+    h, s, v = rgb_to_hsv_channels(r, g, b)
+    hsv = np.stack((h,s,v), axis=-1)
+    return hsv
+
+def hsv_to_rgb(arr):
+    hsv_to_rgb_channels = np.vectorize(colorsys.hsv_to_rgb)
+    h, s, v = np.rollaxis(arr, axis=-1)
+    r, g, b = hsv_to_rgb_channels(h, s, v)
+    rgb = np.stack((r,g,b), axis=-1)
+    return rgb
 
 def download_url_to_file(url, dst, progress=True):
     r"""Download object at the given URL to a local path.
@@ -89,7 +93,7 @@ def distance_to_boundary(masks):
                 sr,sc = si
                 mask = (masks[sr, sc] == (i+1)).astype(np.uint8)
                 contours = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_NONE)
-                pvc, pvr = np.concatenate(contours[0], axis=0).squeeze().T  
+                pvc, pvr = np.concatenate(contours[-2], axis=0).squeeze().T  
                 ypix, xpix = np.nonzero(mask)
                 min_dist = ((ypix[:,np.newaxis] - pvr)**2 + 
                             (xpix[:,np.newaxis] - pvc)**2).min(axis=1)
@@ -141,13 +145,13 @@ def masks_to_outlines(masks):
             outlines[i] = masks_to_outlines(masks[i])
         return outlines
     else:
-        slices = find_objects(masks)
+        slices = find_objects(masks.astype(int))
         for i,si in enumerate(slices):
             if si is not None:
                 sr,sc = si
                 mask = (masks[sr, sc] == (i+1)).astype(np.uint8)
                 contours = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_NONE)
-                pvc, pvr = np.concatenate(contours[0], axis=0).squeeze().T            
+                pvc, pvr = np.concatenate(contours[-2], axis=0).squeeze().T            
                 vr, vc = pvr + sr.start, pvc + sc.start 
                 outlines[vr, vc] = 1
         return outlines
@@ -159,7 +163,7 @@ def outlines_list(masks):
         mn = masks==n
         if mn.sum() > 0:
             contours = cv2.findContours(mn.astype(np.uint8), mode=cv2.RETR_EXTERNAL, method=cv2.CHAIN_APPROX_NONE)
-            contours = contours[0]
+            contours = contours[-2]
             cmax = np.argmax([c.shape[0] for c in contours])
             pix = contours[cmax].astype(int).squeeze()
             if len(pix)>4:
@@ -194,7 +198,7 @@ def get_mask_perimeters(masks):
         mn = masks==(n+1)
         if mn.sum() > 0:
             contours = cv2.findContours(mn.astype(np.uint8), mode=cv2.RETR_EXTERNAL,
-                                        method=cv2.CHAIN_APPROX_NONE)[0]
+                                        method=cv2.CHAIN_APPROX_NONE)[-2]
             #cmax = np.argmax([c.shape[0] for c in contours])
             #perimeters[n] = get_perimeter(contours[cmax].astype(int).squeeze())
             perimeters[n] = np.array([get_perimeter(c.astype(int).squeeze()) for c in contours]).sum()
@@ -381,19 +385,4 @@ def fill_holes_and_remove_small_masks(masks, min_size=15):
                     msk = binary_fill_holes(msk)
                 masks[slc][msk] = (j+1)
                 j+=1
-    return masks
-
-def check_mkl():
-    print('Running test snippet to check if MKL running (https://mxnet.apache.org/versions/1.6/api/python/docs/tutorials/performance/backend/mkldnn/mkldnn_readme.html#4)')
-    process = subprocess.Popen(['python', 'test_mkl.py'],
-                                stdout=subprocess.PIPE, stderr=subprocess.PIPE, 
-                                cwd=os.path.dirname(os.path.abspath(__file__)))
-    stdout, stderr = process.communicate()
-    if len(stdout)>0:
-        print('** MKL version working - CPU version is fast. **')
-        mkl_enabled = True
-    else:
-        print('WARNING: MKL version not working/installed - CPU version will be SLOW!')
-        mkl_enabled = False
-    return mkl_enabled
-        
\ No newline at end of file
+    return masks
\ No newline at end of file
